@@ -42,13 +42,13 @@ class Call:
 
     `skip` skip calls when a hash of the same parameters already exists in sqlite (`True`)
 
-    `clear` remove existing data before calls (sqlite table or `data/raw/`) (`False`)
+    `clear` remove existing data before calls (sqlite or `data/raw/`) (`False`)
 
     `server` select a server from `config.yml` (`"ske"`)
 
-    `threads` for asynchronous calling (`None` for default, otherwise an integer)
+    `threads` for asynchronous calling (`None` for default, otherwise an integer <= 32)
 
-    `loglevel` (`"info"`) outputs to `.sgex.log`"""
+    `loglevel` (`"warning"`) outputs to `.sgex.log`"""
 
     def _get_config(self):
         """Gets servers and credentials config file.
@@ -83,14 +83,13 @@ class Call:
         Sketch Grammar Explorer
 
         No config file detected: generating 'config.yml' - add credentials, then try again."
-
         If a server requires credentials, add 'username' and 'api_key' to server info.
-        API keys can also be managed in the OS keyring with these commands:
+        API keys can also be managed with the `keyring` package using the format below:
 
-            sgex.config.keyring_add_key()
-            sgex.config.keyring_delete_key()
+            import keyring
+            keyring.set_password("server","username","api_key")
 
-        If using keyring to store an API key, leave 'api_key' empty or set as null.
+        In this case, include `username` in the config file and leave `api_key` as `null`.
 
         See documentation at https://github.com/engisalor/sketch-grammar-explorer"""
             )
@@ -272,6 +271,7 @@ class Call:
         logger.debug(f'RESPONSE {packet["response"].status_code}: {packet["item"]["id"]}')
 
         if self.format == "json":
+            response_json = packet["response"].json()
             # Keep data
             if "keep" in packet["item"]["params"]:
                 keep = packet["item"]["params"]["keep"]
@@ -284,11 +284,11 @@ class Call:
                         f'Bad type for "keep" {type(keep)}: use string, list, tuple'
                     )
                 kept = {
-                    k: v for k, v in packet["response"].json().items() if k in keeps
+                    k: v for k, v in response_json.items() if k in keeps
                 }
                 self.data = kept
             else:
-                self.data = packet["response"].json()
+                self.data = response_json
             # Scrub credentials
             if "request" in self.data:
                 for i in ["api_key", "username"]:
@@ -297,8 +297,8 @@ class Call:
             
             # API errors
             error = None
-            if "error" in packet["response"].json():
-                error = packet["response"].json()["error"]
+            if "error" in response_json:
+                error = response_json["error"]
                 self.errors.append(error)
                 logger.warning(f'{packet["item"]["id"]} {error}')
 
@@ -312,15 +312,22 @@ class Call:
                 else:
                     meta = packet["item"]["params"]["meta"]
 
+            versions = {
+               "api_version": response_json.get("api_version"),
+               "manatee_version": response_json.get("manatee_version"),
+               "sgex_version": sgex.__version__,
+            }
+
             # Write to db
             self.c.execute(
-                "INSERT OR REPLACE INTO calls VALUES (?,?,?,?,?,?,?,?,?)",
+                "INSERT OR REPLACE INTO calls VALUES (?,?,?,?,?,?,?,?,?,?)",
                 (
                     self.input,
                     packet["item"]["params"]["type"],
                     packet["item"]["id"],
                     packet["item"]["params"]["hash"],
                     self.timestamp,
+                    json.dumps(versions, sort_keys=True),
                     json.dumps(packet["item"]["params"]["call"], sort_keys=True),
                     meta,
                     error,
@@ -389,11 +396,12 @@ class Call:
     def _make_table(self):
         self.c.execute(
             """CREATE TABLE IF NOT EXISTS calls (
-            input,
+            input text,
             type text,
             id text,
             hash text PRIMARY KEY,
             timestamp text,
+            version text,
             call text,
             meta text,
             error text,
@@ -407,6 +415,7 @@ class Call:
             "input      ": self.input,
             "output     ": self.output,
             "format     ": self.format,
+            "length     ": self.length,
             "queued     ": self.queued,
             "skip       ": self.skip,
             "clear      ": self.clear,
@@ -493,7 +502,8 @@ class Call:
         self._pre_calls()
         manifest = self._make_manifest(credentials)
         self.queued = len(manifest)
-        logger.info(f"QUEUED {self.queued} / {len(self.calls)}")
+        self.length = len(self.calls)
+        logger.info(f"QUEUED {self.queued} / {self.length}")
 
         # Call mode
         if server_info.get("asynchronous"):
@@ -522,7 +532,7 @@ class Call:
             self.conn.close()
 
         if self.errors:
-            logger.warning(f"{len(self.errors)} API error(s): {set(self.errors)}")
+            logger.info(f"{len(self.errors)} API error(s): {set(self.errors)}")
         
         t1 = time.perf_counter()
         logger.info(f"CALLED {self.called} in {t1 - t0:0.2f} secs")
